-- List of frame name letters.
local frame_list = { 'A', 'B', 'C', 'D', 'E', 'F', 'G', 'H', 'I', 'J', 'K', 'L', 'M', 'N', 'O', 'P', 'Q', 'R', 'S', 'T', 'U', 'V', 'W', 'X', 'Y', 'Z', '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l', 'm', 'n', 'o', 'p', 'q', 'r', 's', 't', 'u', 'v', 'w', 'x', 'y', 'z', '!', '@' }

-- Register new animated billboard sprites.
local billboard_register = function(...)
	for _, billboard in ipairs{...} do
		local thingtype = billboard[1]
		local name = billboard[2]
		local sprite = billboard[3]
		local frames = billboard[4]
		local fps = billboard[5]
		
		if type(fps) == 'number' then
			-- Treat single FPS N as full list { N, N, N, N, N... }
			local list = {}
			
			for i, frames in ipairs(frames) do
				list[i] = fps
			end
			
			fps = list
		end
	
		-- Register mob type, sprite and state slots.
		local freeslots = { 'MT_' .. name, 'SPR_' .. sprite }
	
		for i in ipairs(frames) do
			freeslots[#freeslots + 1] = 'S_' .. name .. frame_list[i]
		end
	
		freeslot(unpack(freeslots))
	
		mobjinfo[_G['MT_' .. name]] = {
			doomednum = thingtype,
			spawnstate = _G['S_' .. name .. 'A'],
			spawnhealth = 1000,
			radius = 8 * FRACUNIT,
			height = 52 * FRACUNIT,
			flags = MF_NOBLOCKMAP | MF_NOCLIP | MF_SCENERY
		}
	
		-- Register a state for each frame.
		for i, frame in ipairs(frames) do
			states[_G['S_' .. name .. frame_list[i]]] = {
				sprite = _G['SPR_' .. sprite],
				frame = _G[frame_list[frame]],
				tics = 35 / fps[i],
				var1 = 5,
				var2 = 4,
				nextstate = _G['S_' .. name .. (i == #frames and 'A' or frame_list[i + 1])]
			}
		end
	end
end

billboard_register(
	{ 6701, 'CROWDMEMBERARMS',				'MBAR', { 1, 2, 3, 4, 5, 6 }, 8 },
	{ 6702, 'CROWDMEMBERCOCONUTS',			'MBCN', { 1, 2, 1, 2, 1, 2, 1, 3, 4, 5, 4, 5, 4, 5, 3 }, 8 },
	{ 6703, 'CROWDMEMBERDAVYSPROCKET',		'MBDS', { 1, 2, 3, 4, 3, 2 }, { 4, 8, 8, 4, 8, 8 } },
	{ 6704, 'CROWDMEMBERDRROBOTNIK',		'MBDR', { 1, 2, 3, 4, 5, 6, 7, 8, 7, 9, 10, 11, 12, 13, 12 }, { 2, 8, 8, 8, 8, 2, 8, 2, 8, 2, 8, 8, 8, 1, 8 } },
	{ 6705, 'CROWDMEMBERDRAGONBREATH',		'MBDB', { 1, 2, 3, 2 }, 4 },
	{ 6706, 'CROWDMEMBERDYNAMIGHT',			'MBDM', { 1, 2, 3, 4, 5, 6, 7, 8, 3, 2 }, { 4, 12, 12, 12, 12, 12, 12, 12, 12, 12 } },
	{ 6707, 'CROWDMEMBERFRANKLY',			'MBFR', { 1, 2, 3, 4, 3, 2 }, 8 },
	{ 6708, 'CROWDMEMBERGROUNDER',			'MBGR', { 1, 2, 1, 2, 1, 2 }, { 4, 8, 8, 8, 8, 8} },
	{ 6709, 'CROWDMEMBERHUMPTY',			'MBHM', { 1, 2, 3, 4, 5, 4, 3, 2 }, 8 },
	{ 6710, 'CROWDMEMBERSCRATCH',			'MBSC', { 1, 2, 3, 4, 5, 6, 5, 4, 5, 6, 5, 4, 3, 2 }, { 1, 8, 8, 1, 12, 12, 12, 12, 12, 12, 12, 1, 8, 8 } },
	{ 6711, 'CROWDMEMBERSIRFFUZZYLOGIK',	'MBSF', { 1, 2, 3, 2 }, 8 },
	{ 6712, 'CROWDMEMBERSKWEEL',			'MBSK', { 1, 2, 3, 4, 5, 4, 3, 2 }, 12 },
	{ 6713, 'CROWDMEMBERSPIKE',				'MBSP', { 1, 2, 3, 4, 3, 2 }, { 8, 8, 8, 4, 8, 8 } }
)